/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.jms.connection;

import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.commons.pool.BasePoolableObjectFactory;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;

public abstract class AbstractJMSConnectionFactory extends BasePoolableObjectFactory {

    private static final String JNDI_CONNECTION_FACTORY = "jndi-connection-factory";

    private static final String JNDI_PROVIDER_URL = "jndi-provider-url";

    private static final String JNDI_DESTINATION_NAME = "jndi-destination-name";

    private static final String JNDI_INITIAL_CONTEXT_FACTORY = "jndi-initial-context-factory";

    private static final String USER = "user";

    private static final String PASSWORD = "password";

    private static final String TRANSACTED = "transacted";

    private ConfigurationExtensions extensions;

    private Context context;

    private Logger logger;

    private short contextReferenceCounter;

    public AbstractJMSConnectionFactory(ConfigurationExtensions extensions, Logger logger) {
        super();
        this.logger = logger;
        this.extensions = extensions;
    }

    @Override
    public void activateObject(Object connectionObject) throws Exception {
        logger.log(Level.FINEST, "Activate connection");
        AbstractJMSConnection connection = (AbstractJMSConnection) connectionObject;
        connection.start();
    }

    @Override
    public void passivateObject(Object connectionObject) throws Exception {
        logger.log(Level.FINEST, "Passivate connection");
        // do nothing
    }

    @Override
    public boolean validateObject(Object connectionObject) {
        logger.log(Level.FINEST, "Validate connection");
        boolean result = true;
        AbstractJMSConnection connection = (AbstractJMSConnection) connectionObject;

        try {
            connection.stop();
            connection.start();
        } catch (PEtALSCDKException e) {
            e.printStackTrace();
            result = false;
        }

        return result;
    }

    @Override
    public Object makeObject() throws Exception {
        logger.log(Level.FINEST, "create connection");
        if (contextReferenceCounter == 0) {
            this.context = setupContext(extensions);
        }
        contextReferenceCounter++;

        // create the jmsConnection object that contains all information
        AbstractJMSConnection connection = this.createJmsConnection(context, extensions);

        connection.init();
        return connection;
    }

    @Override
    public void destroyObject(Object obj) throws Exception {
        logger.log(Level.FINEST, "destroy connection");

        AbstractJMSConnection connection = (AbstractJMSConnection) obj;
        connection.close();
        contextReferenceCounter--;
        if (contextReferenceCounter == 0) {
            try {
                this.context.close();
            } catch (NamingException e) {
                throw new PEtALSCDKException("Failed to close the JNDI context", e);
            }
        }
    }

    protected abstract AbstractJMSConnection createJMSConnection(Context context, String connName,
            String destName, String user, String password, boolean transacted);

    /**
     * Setup a JNDIContext to access a JMS destination.
     * 
     * @param suRootPath
     *            path where jndi context factory class can be found
     * @param extensions
     *            jndi properties
     * @return a JNDI context initialized with the given jbi.xml properties
     * @throws TechnicalException
     *             if JNDI context cannot be created
     */
    private Context setupContext(ConfigurationExtensions extensions) throws PEtALSCDKException {

        // Create JNDI properties
        Properties jndiProperties = createJNDIProperties(extensions);

        Context initialContext;

        try {
            initialContext = new InitialContext(jndiProperties);
        } catch (NamingException e) {
            throw new PEtALSCDKException("Failed to create JNDI Initial Context : "
                    + e.getMessage());
        }

        // Create the JNDI context
        return initialContext;
    }

    /**
     * extract jndi properties from the jbi.xml extensions
     * 
     * @param extensions
     * @return
     */
    private Properties createJNDIProperties(ConfigurationExtensions extensions) {
        Properties outProperties = new Properties();

        String url = extensions.get(JNDI_PROVIDER_URL);
        String factory = extensions.get(JNDI_INITIAL_CONTEXT_FACTORY);

        outProperties.setProperty(InitialContext.PROVIDER_URL, url);
        outProperties.setProperty(InitialContext.INITIAL_CONTEXT_FACTORY, factory);
        return outProperties;
    }

    /**
     * Setup a JMS connection to access a JMS destination.
     * 
     * @param context
     *            jndi context where the connection factory and the destination
     *            can be found
     * @param extensions
     *            contains connectionFactory and destination names, user/pwd,
     *            transacted and acknowledge modes.
     * @return a JMSConnection in a stopped state.
     * @throws PEtALSCFException
     *             a mandatory property is not found in the JBI.XML descriptor
     */
    private AbstractJMSConnection createJmsConnection(Context context,
            ConfigurationExtensions extensions) throws PEtALSCDKException {
        String connName = extensions.get(JNDI_CONNECTION_FACTORY);

        String destName = extensions.get(JNDI_DESTINATION_NAME);

        String user = extensions.get(USER);

        String password = extensions.get(PASSWORD);

        boolean transacted = false;
        String transac = extensions.get(TRANSACTED);
        transacted = Boolean.valueOf(transac);

        return this.createJMSConnection(context, connName, destName, user, password, transacted);

    }

}
