/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: JMSMessageListener.java 12:18:02 alouis $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.jms.listener;


import java.util.Enumeration;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jms.BytesMessage;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.jms.TextMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.apache.commons.lang.CharEncoding;
import org.ow2.petals.bc.jms.Constants;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractExternalListener;
import org.ow2.petals.component.framework.util.UtilFactory;

import static org.ow2.petals.bc.jms.Constants.Extensions.OPERATION;
import static org.ow2.petals.bc.jms.Constants.JMSPropertyNames.JMS_OPERATION_NAME;

/**
 * This class is the JMS message listener for the JmsBC component
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.1
 * @author alouis,ofabre
 */
public class JMSMessageListener implements MessageListener {

    private final AbstractExternalListener externalListener;

    private final Logger logger;

    public JMSMessageListener(final AbstractExternalListener externalListener) {
        this.externalListener = externalListener;
        this.logger = externalListener.getLogger();
    }

    public void onMessage(final Message jmsExchange) {
        this.logger.log(Level.FINEST, "onMessage");

        final ConfigurationExtensions extensions = externalListener.getExtensions();
        this.logger.log(Level.FINEST, "A JMS message is received from '"
                + extensions.get("jndi-provider-url") + "'.");

        try {
            // 1. Only TextMessage, BytesMessage or Object Message are
            // supported.
            if (!(jmsExchange instanceof TextMessage || jmsExchange instanceof BytesMessage || jmsExchange instanceof ObjectMessage)) {
                this.logger.log(Level.SEVERE,
                        "Only TextMessage, BytesMessage or ObjectMessage are supported."
                                + jmsExchange.getClass().getName());
            }

            // 2. Create an exchange
            Exchange exchange;
            try {
                exchange = this.externalListener.createConsumeExchange();
                if (exchange == null) {
                    // try to create an in-only message if an invalid pattern is
                    // specified in consumes extensions
                    this.logger
                            .log(Level.WARNING,
                                    "An invalid pattern is specified in consumes extensions. So an in-only message is sent.");
                    exchange = this.externalListener.createConsumeExchange(externalListener
                            .getConsumes(), MEPConstants.IN_ONLY_PATTERN);
                }
            } catch (final MessagingException e) {
                // try to create an in-only message if no pattern specified in
                // consumes extensions
                this.logger
                        .log(Level.WARNING,
                                "No pattern specified in consumes extensions. So an in-only message is sent.");
                exchange = this.externalListener.createConsumeExchange(externalListener
                        .getConsumes(), MEPConstants.IN_ONLY_PATTERN);
            }

            // 3. transform jmsExchange->normalizedMessage

            if (jmsExchange instanceof TextMessage) {
                this.logger.log(Level.FINEST, "It's a TextMessage.");
                final TextMessage textMessage = (TextMessage) jmsExchange;
                this.transform(textMessage, exchange);
            } /*else if (jmsExchange instanceof BytesMessage) {
                this.logger.log(Level.FINEST, "It's a BytesMessage.");
                final BytesMessage byteMessage = (BytesMessage) jmsExchange;
                this.transform(byteMessage, exchange);
            } else if (jmsExchange instanceof ObjectMessage) {
                this.logger.log(Level.FINEST, "It's a ObjectMessage.");
                final ObjectMessage objectMessage = (ObjectMessage) jmsExchange;
                transform(objectMessage, exchange);
            }*/

            // 4. send the exchange, use the framework's send() method
            this.externalListener.send(exchange);

            this.logger.log(Level.FINEST, "The message has been sent.");

        } catch (final ClassCastException e) {
            this.logger.log(Level.SEVERE,
                    "Only TextMessage are processed by petals-binding-jms component.", e);
        } catch (final Exception e) {
            this.logger.log(Level.SEVERE, e.getClass() + " " + e.getMessage(), e);
        }

    }

    /**
     * Transform the TextMessage into a NormalizedMessage of the JBI MsgExch .
     * The content and properties are set.
     * 
     * @param jmsMessage
     * @param exchange
     * @throws PEtALSComponentSDKException
     *             can not create a Source object from the jms text content
     * @throws JMSException
     *             getting jms content or properties
     * @throws MessagingException
     *             setting NormalizedMessage properties or content
     * @throws PEtALSCDKException
     */
    private void transform(final TextMessage jmsMessage, final Exchange exchange)
            throws JMSException, MessagingException, PEtALSCDKException {
        this.logger.log(Level.FINE, "transform");

        // set content
        final String content = jmsMessage.getText();
        final Source source = UtilFactory.getSourceUtil().createSource(content, CharEncoding.UTF_8);

        exchange.getInMessage().setContent(source);

        // set the operation in the JBI message exchange
        this.setOperation(jmsMessage, exchange);

        // set remaining properties of the JMS message into the JBI message
        // exchange normalized message "IN"
        this.setMessageProperties(jmsMessage, exchange.getInMessage());
    }

    /**
     * Transform the BytesMessage into a NormalizedMessage of the JBI MsgExch .
     * The content and properties are set.
     * 
     * @param jmsMessage
     * @param exchange
     * @throws PEtALSComponentSDKException
     *             can not create a Source object from the jms byte content
     * @throws JMSException
     *             getting jms content or properties
     * @throws MessagingException
     *             setting NormalizedMessage properties or content
     * @throws PEtALSCDKException
     */
    /*private void transform(final BytesMessage jmsMessage, final Exchange exchange)
            throws JMSException, MessagingException, PEtALSCDKException {

        this.logger.log(Level.FINE, "transform");

        // set content
        final byte[] content = new byte[(int) jmsMessage.getBodyLength()];
        jmsMessage.readBytes(content);

        // create a streamDataHandler with the byteArray
        final ByteArrayInputStream input = new ByteArrayInputStream(content);
        final Source sourceContent = new StreamSource(input);
        exchange.setInMessageContent(sourceContent);

        // set the operation in the JBI message exchange
        this.setOperation(jmsMessage, exchange);

        // set remaining properties of the JMS message into the JBI message
        // exchange normalized message "IN"
        this.setMessageProperties(jmsMessage, exchange.getInMessage());

    }*/

    /**
     * Transform the ObjectMessage into a NormalizedMessage of the JBI MsgExch .
     * The content and properties are set.
     * 
     * @param jmsMessage
     * @param exchange
     * @throws PEtALSComponentSDKException
     *             can not create an attachment object from the jms content
     * @throws JMSException
     *             getting jms content or properties
     * @throws MessagingException
     *             setting NormalizedMessage properties or content
     * @throws PEtALSCDKException
     * @throws IOException
     *             serilization exception
     */
    /*@Deprecated
    private void transform(final ObjectMessage jmsMessage, final Exchange exchange)
            throws JMSException, MessagingException, PEtALSCDKException, IOException {
        this.logger.log(Level.FINE, "transform");
        // get the object as a byteArray
        final ByteArrayOutputStream out = new ByteArrayOutputStream();
        final ObjectOutputStream objectOut = new ObjectOutputStream(out);
        objectOut.writeObject(jmsMessage.getObject());
        objectOut.flush();
        final byte[] serializedObject = out.toByteArray();
        objectOut.close();

        // create a streamDataHandler with the byteArray
        final ByteArrayInputStream input = new ByteArrayInputStream(serializedObject);
        final String className = jmsMessage.getObject().getClass().getCanonicalName();
        final StreamDataSource sds = new StreamDataSource(input, className);
        exchange.getInMessage().addAttachment(className, new DataHandler(sds));
        final String content = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><info>Object set as attachment : "
                + className + "</info>";
        final Source source = UtilFactory.getSourceUtil().createSource(content, CharEncoding.UTF_8);

        exchange.getInMessage().setContent(source);

        // set the operation in the JBI message exchange
        this.setOperation(jmsMessage, exchange);

        // set remaining properties of the JMS message into the JBI message
        // exchange normalized message "IN"
        this.setMessageProperties(jmsMessage, exchange.getInMessage());
    }*/

    /**
     * <p>
     * Set the operation into the JBI message exchange according to the
     * following priorized values:
     * </p>
     * 
     * @param jmsMessage
     *            The JMS message from which the operation is extracted.
     * @param exchange
     *            The JBI message exchange in which the operation should be set.
     * @throws JMSException
     */
    private void setOperation(final Message jmsMessage, final Exchange exchange)
            throws JMSException {
        if (jmsMessage.getStringProperty(JMS_OPERATION_NAME) != null
                && jmsMessage.getStringProperty(JMS_OPERATION_NAME).length() > 0) {

            exchange.setOperation(QName.valueOf(jmsMessage
                    .getStringProperty(JMS_OPERATION_NAME)));
        } else if (this.externalListener.getExtensions().get(OPERATION) != null) {
            exchange.setOperation(QName.valueOf(this.externalListener.getExtensions()
                    .get(OPERATION)));
        }
    }

    /**
     * <p>
     * Copy properties from the JMS message into the JBI message exchange
     * normalized message, except the following ones:
     * <ul>
     * <li>
     * <code>{@value Constants.JMSPropertyNames#JMS_PROPERTY_NAME_SOAPACTION}<code>:
     * that is moved to the operation of the JBI message exchange.</li>
     * </ul>
     * </p>
     * 
     * @param jmsMessage
     *            The JMS message from which the properties are extracted.
     * @param normalizedMessage
     *            The JBI message exchange normalized message from which the
     *            properties are extracted.
     * @throws JMSException
     */
    private void setMessageProperties(final Message jmsMessage,
            final NormalizedMessage normalizedMessage) throws JMSException {
        for (final Enumeration<?> it = jmsMessage.getPropertyNames(); it.hasMoreElements();) {
            final String jmsPropertyName = (String) it.nextElement();

            if (!JMS_OPERATION_NAME.equals(jmsPropertyName)) {
                normalizedMessage.setProperty(jmsPropertyName, jmsMessage
                        .getObjectProperty(jmsPropertyName));
            }
        }
    }
}
