/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.jms.listener;

import java.io.IOException;
import java.util.Iterator;
import java.util.logging.Level;

import javax.jbi.messaging.NormalizedMessage;
import javax.jms.JMSException;
import javax.jms.JMSSecurityException;
import javax.jms.Message;
import javax.jms.TextMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.TransformerException;

import org.apache.commons.pool.impl.GenericObjectPool;
import org.ow2.petals.bc.jms.JmsBC;
import org.ow2.petals.bc.jms.connection.JMSProviderConnection;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.UtilFactory;

import static org.ow2.petals.bc.jms.Constants.JMSPropertyNames.JMS_OPERATION_NAME;

public class ServiceManager {

    private AbstractJBIListener listener;

    public ServiceManager(AbstractJBIListener listener) {
        super();
        this.listener = listener;
    }

    public void onJBIMessage(Exchange exchange) throws PEtALSCDKException {
        if (MEPConstants.IN_ONLY_PATTERN.value().equals(exchange.getPattern())) {
            sendJMSMessage(exchange);
        } else {
            throw new PEtALSCDKException("MessageExchangePattern not allowed :"
                    + exchange.getPattern());
        }
    }

    /**
     * Send a JMS message to the JMS queue corresponding to the specified
     * exchange's endpoint.
     * 
     * @param exchange
     * @throws TechnicalException
     *             no JMS destination found, or problem while sending the
     *             message to JMS
     * 
     */
    protected void sendJMSMessage(Exchange exchange) throws PEtALSCDKException {
        listener.getLogger().log(Level.FINE, "sendJMSMessage");

        // find the connection to use for this exchange
        JmsBC component = (JmsBC) listener.getComponent();
        GenericObjectPool pool = component.getJmsProducerConnections().get(listener.getProvides());
        JMSProviderConnection conn;
        try {
            conn = (JMSProviderConnection) pool.borrowObject();
        } catch (Exception e1) {
            String exceptionMsg = "Can't retrieve a JMS connection "
                    + "linked to the following endpoint: " + exchange.getEndpoint();
            throw new PEtALSCDKException(exceptionMsg, e1);
        }
        try {
            if (conn == null) {
                // No such endpoint registered
                throw new PEtALSCDKException("Can't retrieve a JMS connection "
                        + "linked to the following endpoint: " + exchange.getEndpoint());
            } else {
                listener.getLogger().log(
                        Level.INFO,
                        "Try to send JBI message to '" + conn.getDestinationName()
                                + "' JMS provider.");
            }

            // transform the jbi msg into a jms msg
            // if an attachment is found in the exchange,
            // assume that we have to send a jms objectMessage
            Message msg = null;
            try {
                if (exchange.getInMessage().getAttachmentNames().size() == 0) {
                    TextMessage textMsg = null;
                    textMsg = conn.getSession().createTextMessage();
                    transform(exchange, textMsg);
                    msg = textMsg;
                } /*else if (exchange.getInMessage().getAttachmentNames()
                        .contains("JMS_BytesMessage")) {
                    BytesMessage bytesMessage = null;
                    bytesMessage = conn.getSession().createBytesMessage();
                    transform(exchange, bytesMessage);
                    msg = bytesMessage;
                } else {
                    ObjectMessage objectMsg = null;
                    objectMsg = conn.getSession().createObjectMessage();
                    transform(exchange, objectMsg);
                    msg = objectMsg;
                }*/
            } catch (Exception e) {
                listener.getLogger().log(Level.SEVERE,
                        "Can not create the JMS message from JBI message.", e);
                if (e instanceof PEtALSCDKException) {
                    throw (PEtALSCDKException) e;
                } else {
                    throw new PEtALSCDKException(
                            "Can not create the JMS message from JBI message.", e);
                }
            }

            // send
            try {
                conn.getMessageProducer().send(msg);
                if (conn.isTransacted()) {
                    conn.getSession().commit();
                }
            } catch (JMSSecurityException e) {
                String exceptionMsg = "Can not send the JMS message, WRITE right not granted.";
                throw new PEtALSCDKException(exceptionMsg, e);
            } catch (JMSException e) {
                String exceptionMsg = "Can not send the JMS message.";
                throw new PEtALSCDKException(exceptionMsg, e);
            }
        } finally {
            try {
                pool.returnObject(conn);
            } catch (Exception e) {
                String exceptionMsg = "Can't free a JMS connection of the pool.";
                throw new PEtALSCDKException(exceptionMsg, e);
            }
        }
        listener.getLogger()
                .log(Level.INFO, "Message sent to '" + conn.getDestinationName() + "'.");
    }

    /**
     * Transform the NormalizedMessage of a JBI MsgExch into a TextMessage. The
     * content , properties of the IN NormalizedMessage are set.
     * 
     * @param exchange
     * @param jmsMessage
     * @throws JMSException
     *             writing properties to the TextMessage
     * @throws TransformerException
     *             NormalizedMessage content access error
     * @throws IOException
     *             NormalizedMessage content access error
     * @throws PEtALSCDKException
     */
    private void transform(final Exchange exchange, final TextMessage jmsMessage)
            throws JMSException, TransformerException, IOException, PEtALSCDKException {
        listener.getLogger().log(Level.FINE, "transform");

        NormalizedMessage in = exchange.getInMessage();

        // TODO: Do we attach the global properties of the exchange?

        // set normalized message properties
        this.setMessageProperties(jmsMessage, in);

        // set the SOAP Action operation in the JMS message
        this.setServiceOperation(exchange, jmsMessage);

        // set content
        String inSourceAsString = UtilFactory.getSourceUtil().createString(in.getContent());
        jmsMessage.setText(inSourceAsString);
    }

    /**
     * Transform the NormalizedMessage of a JBI MsgExch into a BytesMessage. The
     * content , properties of the IN NormalizedMessage are set.
     * 
     * @param exchange
     * @param jmsMessage
     * @throws JMSException
     *             writing properties to the BytesMessage
     * @throws TransformerException
     *             NormalizedMessage content access error
     * @throws IOException
     *             NormalizedMessage content access error
     * @throws PEtALSCDKException
     */
    /*private void transform(final Exchange exchange, final BytesMessage jmsMessage)
            throws JMSException, TransformerException, IOException, PEtALSCDKException {
        listener.getLogger().log(Level.FINE, "transform");

        NormalizedMessage in = exchange.getInMessage();

        // TODO: Do we attach the global properties of the exchange?

        // set normalized message properties
        this.setMessageProperties(jmsMessage, in);

        // set the SOAP Action operation in the JMS message
        this.setSOAPAction(exchange, jmsMessage);

        // retrieve the binary object
        Source sourceContent = in.getContent();
        ObjectInputStream objectInput = null;
        if (sourceContent instanceof StreamSource)
            objectInput = new ObjectInputStream(((StreamSource) sourceContent).getInputStream());
        int readByte = objectInput.read();
        while (readByte != -1) {
            jmsMessage.writeByte((byte) readByte);
            readByte = objectInput.read();
        }
    }*/

    /**
     * Transform the NormalizedMessage of a JBI MsgExch into an ObjectMessage.
     * The content , properties of the IN NormalizedMessage are set. Assume that
     * there is an attachment in the exchange
     * 
     * @param exchange
     * @param jmsMessage
     * @throws JMSException
     *             writing properties to the TextMessage
     * @throws TransformerException
     *             NormalizedMessage content access error
     * @throws IOException
     *             NormalizedMessage content access error
     * @throws ClassNotFoundException
     *             class of the object is not found in the classLoader
     */
   /* @Deprecated
    private void transform(final Exchange exchange, final ObjectMessage jmsMessage)
            throws JMSException, TransformerException, IOException, ClassNotFoundException {
        listener.getLogger().log(Level.FINE, "transform");

        NormalizedMessage in = exchange.getInMessage();

        // TODO: Do we attach the global properties of the exchange?

        // set normalized message properties
        this.setMessageProperties(jmsMessage, in);

        // set the SOAP Action operation in the JMS message
        this.setSOAPAction(exchange, jmsMessage);

        // retrieve the binary object
        String attachmentName = (String) exchange.getInMessage().getAttachmentNames().iterator()
                .next();
        DataHandler dh = exchange.getInMessage().getAttachment(attachmentName);
        ObjectInputStream objectInput = new ObjectInputStream(dh.getInputStream());
        Serializable object = (Serializable) objectInput.readObject();
        jmsMessage.setObject(object);
    }*/

    /**
     * <p>
     * Set the operation name to the JMS message.
     * </p>
     * 
     * @param exchange
     *            The JBI message exchange from which the operation is
     *            extracted.
     * @param jmsMessage
     *            The JMS message in which the SOAP action should be set
     * @throws JMSException
     */
    private void setServiceOperation(final Exchange exchange, final Message jmsMessage)
            throws JMSException {
        final QName operation = exchange.getOperation();
        if (operation != null) {
            jmsMessage.setStringProperty(JMS_OPERATION_NAME, operation.toString());
        }
    }

    /**
     * <p>
     * Copy properties from the JBI message exchange normalized message into the
     * JMS message.
     * </p>
     * 
     * @param normalizedMessage
     *            The JBI message exchange normalized message from which the
     *            properties are extracted.
     * @param jmsMessage
     *            The JMS message in which the properties are set.
     * @throws JMSException
     */
    private void setMessageProperties(final Message jmsMessage,
            final NormalizedMessage normalizedMessage) throws JMSException {
        for (final Iterator<?> it = normalizedMessage.getPropertyNames().iterator(); it.hasNext();) {
            final String propertyName = (String) it.next();
            final Object propertyValue = normalizedMessage.getProperty(propertyName);
            jmsMessage.setObjectProperty(propertyName, propertyValue);
        }
    }
}
